@@ -166,7 +166,7 @@ def output_fc_lags(origin_id,fcdate):
 
 def output_hc_lags(origin_id,fcdate):
     '''
-    Given origin_id, outputted the best dates for lagged reforecast
+    Given origin_id, outputted the best lags for downloading reforecasts
     '''
     if origin_id not in variable_dict.rf_lag_info:
         raise ValueError(f"[ERROR] No forecast lags found for origin_id '{origin_id}'.")
@@ -175,9 +175,11 @@ def output_hc_lags(origin_id,fcdate):
     weekday = fcdate_obj.weekday()+1  # Monday = 1, ..., Sunday = 7
     dayofmonth = fcdate_obj.day
 
-    if isinstance(variable_dict.rf_lag_info[origin_id],list): # DOMs. get closest lags
-        fc_day = fcdate_obj.day
-        rf_freq_info = variable_dict.rf_lag_info[origin_id]
+    lag_info = variable_dict.rf_lag_info[origin_id]
+
+    # ---- Nearest DOM (single closest) -------
+    if 'nearestDOM' in lag_info: # DOMs. get closest lags
+        rf_freq_info = lag_info['nearestDOM']
 
         # Build candidate reforecast dates in prev/current/next month
         candidates = []
@@ -195,7 +197,38 @@ def output_hc_lags(origin_id,fcdate):
         closest_rf = min(candidates, key=lambda d: abs((d - fcdate_obj).days))
         lag = (closest_rf - fcdate_obj).days
         return lag
-    elif variable_dict.rf_lag_info[origin_id]=='ThMoTh':
+    # ------ Before/after DOM -----------------
+    if 'before_after_DOM' in lag_info:
+        rf_freq_info = lag_info['before_after_DOM']
+
+        # Build candidate reforecast dates in prev/current/next month
+        candidates = []
+        for offset_month in [-1, 0, 1]:
+            # Month/year shift
+            month = (fcdate_obj.month - 1 + offset_month) % 12 + 1
+            year = fcdate_obj.year + ((fcdate_obj.month - 1 + offset_month) // 12)
+            for dom in rf_freq_info:
+                try:
+                    candidates.append(datetime(year, month, dom))
+                except ValueError:
+                    continue  # skip invalid dates (e.g., Feb 30)
+
+        lags = [(c-fcdate_obj).days for c in candidates]
+
+        neg_lags = [l for l in lags if l < 0]
+        pos_lags = [l for l in lags if l >= 0]
+
+        largest_neg = max(neg_lags) if neg_lags else None
+        smallest_pos = min(pos_lags) if pos_lags else None
+
+        return [largest_neg,smallest_pos] # before and after lags
+    # ------ daily reforecast initialisations ------
+    if 'daily_lagged' in lag_info:
+        return lag_info['daily_lagged']
+
+    # ------ Weekday based (i.e. only Monday and Thursday reforecasts)-----
+    if 'weekday_based' in lag_info:
+        mode = lag_info['weekday_based']
         # two options depending on whether fcdate is a Monday or Thursday
         if weekday == 4: # Thursday
             return [-3,0,4]
@@ -203,26 +236,50 @@ def output_hc_lags(origin_id,fcdate):
             return [-4,0,3]
         else:
             raise ValueError(f"[ERROR] For origin_id '{origin_id}' forecasts are only every Monday and Thursday, therefore a Monday or Thursday forecast date must be selected.")
-    elif variable_dict.rf_lag_info[origin_id]=='odddates':
-        if (dayofmonth % 2 == 0) or (dayofmonth == 29 and fcdate_obj.month() == 2):
-            return [-1,1] # for even fcdates (or 29th Feb) chosen rfdate before and after forecast date.
-        else:
-            return [-2,0] # for odd dates, choosen current day, minus 2. 
-    elif variable_dict.rf_lag_info[origin_id] == 'CNRevery5days':
-        # make an array of dates from 2020-01-01 to 2020-12-27
-        DOM = {1:[1,6,11,16,21,26,31],2:[5,10,15,20,25],3:[2,7,12,17,22,27],4:[1,6,11,16,21,26],5:[1,6,11,16,21,26,31],6:[5,10,15,20,25,30],7:[5,10,15,20,25,30],8:[4,9,14,19,24,29],9:[3,8,13,18,23,28],10:[3,8,13,18,23,28],11:[2,7,12,17,22,27],12:[2,7,12,17,22,27]}
-        CNR_rf_dates = []
-        for month, days in DOM.items():
-            for day in days:
-                CNR_rf_dates.append(datetime(2020,month,day))
-        # change year of fcdate to 2020-fcdate(MM)-fcdate(DD)
-        fc_date_2020 = datetime.strptime(f"2020{fcdate[4:]}",'%Y%m%d')
-        # nearest date to altered fcdate is rf_date
-        closest_day=min(CNR_rf_dates,key=lambda x:abs(fc_date_2020-x))
-        lag = (closest_day - fc_date_2020).days
 
-    else:
-        raise ValueError(f"[ERROR] No forecast lags found for origin_id '{origin_id}'.")
+    # ---- Unique modes  ------
+    if 'unique' in lag_info:
+        mode = lag_info['unique']
+
+        # ECMWF ---- odd day reforecasts.
+        if mode == 'odddates':
+            if (dayofmonth % 2 == 0) or (dayofmonth == 29 and fcdate_obj.month() == 2):
+                return [-1,1] # for even fcdates (or 29th Feb) chosen rfdate before and after forecast date.
+            else:
+                return [-2,0] # for odd dates, choosen current day, minus 2. 
+        if mode == 'CNRevery5days': # roughly five days
+            # make an array of dates from 2020-01-01 to 2020-12-27
+            DOM = {1:[1,6,11,16,21,26,31],2:[5,10,15,20,25],3:[2,7,12,17,22,27],4:[1,6,11,16,21,26],5:[1,6,11,16,21,26,31],6:[5,10,15,20,25,30],7:[5,10,15,20,25,30],8:[4,9,14,19,24,29],9:[3,8,13,18,23,28],10:[3,8,13,18,23,28],11:[2,7,12,17,22,27],12:[2,7,12,17,22,27]}
+            CNR_rf_dates = []
+            for year in (2020,2021):
+                for month, days in DOM.items():
+                    for day in days:
+                        CNR_rf_dates.append(datetime(year,month,day))
+            # change year of fcdate to 2020-fcdate(MM)-fcdate(DD)
+            fc_date_2020 = datetime.strptime(f"2020{fcdate[4:]}",'%Y%m%d')
+            # nearest date to altered fcdate is rf_date
+            closest_day=min(CNR_rf_dates,key=lambda x:abs(fc_date_2020-x))
+            lag = (closest_day - fc_date_2020).days
+            return lag
+        if mode == 'JMAtwicepermonth': # twice per month
+            # make an array of dates from 2020-01-01 to 2020-12-27
+            DOM = {1:[16,31],2:[10,25],3:[12,27],4:[11,26],5:[16,31],6:[15,30],7:[15,30],8:[14,29],9:[13,28],10:[13,28],11:[12,27],12:[12,27]}
+            JMA_rf_dates = []
+            for year in (2019,2020,2021):
+                for month, days in DOM.items():
+                    for day in days:
+                        JMA_rf_dates.append(datetime(year,month,day))
+            # change year of fcdate to 2020-fcdate(MM)-fcdate(DD)
+            fc_date_2020 = datetime.strptime(f"2020{fcdate[4:]}",'%Y%m%d')
+
+            lags = [(c-fc_date_2020).days for c in JMA_rf_dates]
+            neg_lags = [l for l in lags if l < 0]
+            pos_lags = [l for l in lags if l >= 0]
+
+            largest_neg = max(neg_lags) if neg_lags else None
+            smallest_pos = min(pos_lags) if pos_lags else None
+
+            return [largest_neg,smallest_pos] 
 
 def get_hindcast_model_date(origin_id,hcdate):
     ''' Given origin_id, output appropriate date for reforecast dataset. This is the hindcast model version, not the set of reforecast dates.
@@ -260,78 +317,6 @@ def get_hindcast_year_span(origin_id,fcdate):
 
     return rf_years
 
-def get_reforecast_date(origin_id,fcdate):
-    ''' Given origin_id, output appropriate reforecast date. The DOY component of the rf_date is used to produce complete set of reforecast dates (i.e. 20010405, 20020505, 20030405).
-
-    Note: for CNRM and CPTEC, a complete list of rfdates is given due to inconsistent day-of-years across reforecast years. 
-    '''
-    if origin_id not in variable_dict.reforecast_freq:
-        raise ValueError(f"[ERROR] No reforecast freq information found for origin_id '{origin_id}'.")
-
-    rf_freq_info = variable_dict.reforecast_freq[origin_id]
-
-    if rf_freq_info == 'rf_model_date':
-        rf_date = get_hindcast_model_date(origin_id,fcdate)
-    elif rf_freq_info == 'fcdate':
-        rf_date = fcdate # rf frequency is at same as forecast frequency
-    elif isinstance(rf_freq_info,(list,tuple)): #days of month
-        # find nearest day of month
-        fc_day = int(fcdate[6:])
-        closest_day = min(rf_freq_info,key=lambda x:abs(fc_day-x))
-        rf_date = fcdate[:6]+f"{closest_day:02d}"
-    elif rf_freq_info == 'CNRevery5days':
-        # make an array of dates from 2020-01-01 to 2020-12-27
-        DOM = {1:[1,6,11,16,21,26,31],2:[5,10,15,20,25],3:[2,7,12,17,22,27],4:[1,6,11,16,21,26],5:[1,6,11,16,21,26,31],
-                6:[5,10,15,20,25,30],7:[5,10,15,20,25,30],8:[4,9,14,19,24,29],9:[3,8,13,18,23,28],10:[3,8,13,18,23,28],
-                11:[2,7,12,17,22,27],12:[2,7,12,17,22,27]}
-        CNR_rf_dates = []
-        for month, days in DOM.items():
-            for day in days: 
-                CNR_rf_dates.append(datetime(2020,month,day))
-        # change year of fcdate to 2020-fcdate(MM)-fcdate(DD)
-        fc_date_2020 = datetime.strptime(f"2020{fcdate[4:]}",'%Y%m%d')
-        # nearest date to altered fcdate is rf_date
-        closest_day=min(CNR_rf_dates,key=lambda x:abs(fc_date_2020-x))
-        rf_date = closest_day.strftime('%Y%m%d')
-    elif rf_freq_info == 'JMA2permonth':
-        jma_rf_dates=[datetime(2020,1,16),datetime(2020,1,31), # writing out list of all 2020 hindcast dates
-                datetime(2020,2,10),datetime(2020,2,25),
-                datetime(2020,3,12),datetime(2020,3,27),
-                datetime(2020,4,11),datetime(2020,4,26),
-                datetime(2020,5,16),datetime(2020,5,31),
-                datetime(2020,6,15),datetime(2020,6,30),
-                datetime(2020,7,15),datetime(2020,7,30),
-                datetime(2020,8,14),datetime(2020,8,29),
-                datetime(2020,9,13),datetime(2020,9,28),
-                datetime(2020,10,13),datetime(2020,10,28),
-                datetime(2020,11,12),datetime(2020,11,27),
-                datetime(2020,12,12),datetime(2020,12,27)]
-        # change year of fcdate to 2020-fcdate(MM)-fcdate(DD)
-        fc_date_2020 = datetime.strptime(f"2020{fcdate[4:]}",'%Y%m%d')
-        # nearest date to altered fcdate is rf_date
-        closest_day=min(jma_rf_dates,key=lambda x:abs(fc_date_2020-x))
-        rf_date = closest_day.strftime('%Y%m%d')
-    elif rf_freq_info == 'CNRMevery7days':
-        rf_dates = pd.date_range('19921231','20171228',freq='7D')
-        rf_date = []
-        for year in np.arange(1993,2017+1):
-            fc_date_one_year = datetime.strptime(f"{year}{fcdate[4:]}",'%Y%m%d')
-            # nearest date to altered fcdate is rf_date
-            closest_day=min(rf_dates,key=lambda x:abs(fc_date_one_year-x))
-            rf_date.append(closest_day.strftime('%Y%m%d'))
-    elif rf_freq_info == 'CPTECevery7days':
-        rf_dates = pd.date_range('19990106','20181226',freq='7D')
-        rf_date = []
-        for year in np.arange(1999,2018+1):
-            fc_date_one_year = datetime.strptime(f"{year}{fcdate[4:]}",'%Y%m%d')
-            # nearest date to altered fcdate is rf_date
-            closest_day=min(rf_dates,key=lambda x:abs(fc_date_one_year-x))
-            rf_date.append(closest_day.strftime('%Y%m%d'))
-    else:
-        raise ValueError(f"[ERROR] No reforecast freq information found for origin_id '{origin_id}'.")
-
-    return rf_date
-
 def output_formatted_leadtimes(leadtime_hour,fcdate,variable,lag=0,fc_enslags=0):
     # create new fcdate based on lag
     new_fcdate = datetime.strptime(fcdate, '%Y%m%d')+timedelta(days=lag)
@@ -439,9 +424,6 @@ def check_and_output_all_hc_arguments(variable,origin_id,hcdate,rfyears=None):
     # after computing reforecast years, check the chosen set
     argument_check.check_requested_reforecast_years(rfyears,origin_id,hcdate)
 
-    # get the appropriate reforecast date
-    #rf_date = get_reforecast_date(origin_id,fcdate)
-
-    return rf_model_date, rfyears #, rf_date
+    return rf_model_date, rfyears
 
 
