@@ -233,18 +233,18 @@ reforecast_pert_members={
         }
 
 rf_lag_info={
-        "ammc":[1,6,11,16,21,26], # DOMs, just choose closest day.
-        "babj":'ThMoTh', # Thursday, Monday, Thursday, lag depends on chosen fcdate
-        "isac":'CNRevery5days',
+        "ammc":{'nearestDOM': [1,6,11,16,21,26]}, # DOMs, just choose closest day.
+        "babj":{'weekday_based':'ThMoTh'}, # Thursday, Monday, Thursday, lag depends on chosen fcdate
+        "isac":{'unique':'CNRevery5days'},
         #"lfpw":[0],
         #"sbsj":[0],
-        "cwao":'ThMoTh',
-        "ecmf":'odddates', # ecmwf has reforecasts every odd day (expect feb 29th). for odd day: -2 and 0, even day: -1 and 1
-        #"rums":[0],
-        #"anso":[-4,-2,0,2,4],
-        #"rjtd":'before_after', # chosen two lags where reforecast is before and after forecast date
-        #"rksl":'before_after',
-        #"kwbc":[-4,-2,0,2,4],
-        #"egrr":'before_after'
+        "cwao":{'weekday_based':'ThMoTh'},
+        "ecmf":{'unique':'odddates'}, # ecmwf has reforecasts every odd day (expect feb 29th). for odd day: -2 and 0, even day: -1 and 1
+        "rums":{'daily_lagged':[0]},
+        "anso":{'daily_lagged':[-4,-2,0,2,4]},
+        "rjtd":{'unique':'JMAtwicepermonth'}, # chosen two lags where reforecast is before and after forecast date
+        "rksl":{'before_after_DOM':[1,9,17,25]}, # DOMs. will choose before and after day
+        "kwbc":{'daily_lagged':[-4,-2,0,2,4]},
+        "egrr":{'before_after_DOM':[1,9,17,25]}
         }
 
